/**
 * Title:        Channel models
 * Description:  AWGN and BSC channel models
 * Copyright:    Copyright (c) 2001, Ling Li
 * Company:      California Institute of Technology
 * @author Ling Li
 * @email  ling@cs.caltech.edu
 * @version 1.0
 */

#include <assert.h>
#include <math.h>
#include "random.h"
#include "channel.h"

static REAL sigma;

/* Eb/N0 in dB */
REAL ch_setEbN0 (REAL EbN0, REAL coderate)
{
    sigma = sqrt(0.5 / coderate) * exp(-log(10) / 20 * EbN0);
    return sigma;
}


void ch_setSigma (REAL s)
{
    sigma = s;
}


REAL ch_getSigma ()
{
    return sigma;
}


/** AWGN
 *  with binary inputs {0, 1}, return output of an AWGN channel.
 *  r[i] is a vector of length 2 in {-1, +1}
 */
void ch_BIT2AWGN (BIT *y, REAL *r, UINT32 len)
{
    while (len--) {
        assert(*y == 0 || *y == 1);
        *r++ = 1 - 2 * (*y++) + sigma * randn();
    }
}


/** AWGN2BSC
 *  change vectors in {-1, +1} to {0, 1}
 */
void ch_AWGN2BSC (REAL *r, BIT *y, UINT32 len)
{
    while (len--)
        *y++ = (*r++ < 0);
}
