/**
 * Title:        Random Number Generator
 * Copyright:    Copyright (c) 2001, Ling Li
 * Company:      California Institute of Technology
 * @author Ling Li
 * @email  ling@cs.caltech.edu
 * @version 1.0
 */

#include <assert.h>
#include <math.h>
#include "config.h"
#include "random.h"

#define AM      (1.0/(IM))
#define IQ      ((IM)/(IA))
#define IR      ((IM)-(IA)*IQ)
#define NDIV    (1+((IM)-1)/(NTAB))

static INT32 idum = 0;
static INT32 iy;
static INT32 iv[NTAB];

/* Schrage's method to compute (IA*x) % IM */
#define SET_NEXT_SEQ(x)  SET_NEXT_LC(x,IA,IM,IQ,IR)

void rand_setseed (INT32 seed)
{
    int j;

    idum = seed;
    if (idum < 1) idum = 1;

    for (j = NTAB + 7; j >= 0; j--) {
        SET_NEXT_SEQ(idum);

        if (j < NTAB) iv[j] = idum;
    }

    iy = iv[0];
}


INT32 rand_getseed ()
{
    assert(idum > 0);
    return idum;
}


/* Uniform over (0, 1) */
REAL randu ()
{
    int j;

    assert(idum > 0);

    SET_NEXT_SEQ(idum);
    j = iy / NDIV;
    iy = iv[j];
    iv[j] = idum;

    return AM * iy;
}


/* Normal distribution with mean 0, variance 1 */
REAL randn ()
{
    REAL x1, s;
    static REAL x2;
    static int even_call = 0;

    if (even_call) {
        even_call = 0;
        return x2;
    }

    do {
        x1 = 2 * randu() - 1;
        x2 = 2 * randu() - 1;

        s = x1 * x1 + x2 * x2;
    } while (s >= 1.0);
    s = sqrt(-2 * log(s) / s);
    x1 *= s;
    x2 *= s;

    even_call = 1;
    return x1;
}
