#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "random.h"
#include "channel.h"
#include "turbo.h"

UINT16 iters[] = {1, 2, 4, 6, 8, 12, 16};
#define it_num      (sizeof(iters) / sizeof(UINT16))
#define MAX_TRIAL   2500
#define MIN_SNR     0
#define MAX_SNR     3

int main(int argc, char *argv[])
{
    BIT *x, *y[3], *u;
    UINT32 len, i, it;
    REAL *r[3], *pi, *LLR;
    int snr;
    FILE *f, *fpi, *fLLR;
    char *fn, pdfn[255], *dot;

#ifdef TURBO_BCJR
    printf("BCJR method\n");
#else
    printf("Message passing\n");
#endif

#ifdef TURBO_ORIG
    printf("The Original Code in Turbo-Code paper\n");
#else
    printf("The code in the project\n");
#endif

/* (127, 4), (531, 1), (7, 79) */
//    turbo_init(7, 79, 1103);
//    len = 1102;

    turbo_init(25643, 529, 65497);
    len = 65496;

    x = (BIT*) malloc(sizeof(BIT) * len);
    u = (BIT*) malloc(sizeof(BIT) * len);
    for (i = 0; i < 3; i++) {
        y[i] = (BIT*) malloc(sizeof(BIT) * (len+4));
        r[i] = (REAL*) malloc(sizeof(REAL) * (len+4));
        assert(r[i] != NULL);
    }

    x[0] = 1; x[1] = x[2] = x[3] = x[4] = x[5] = 0;

    for (i = 6; i < len; i++)
        x[i] = x[i-5] ^ x[i-6];

    turbo_encode(x, y, len, 4);

    rand_setseed(18013);

    LLR = (REAL*) malloc(sizeof(REAL) * len);
    pi = (REAL*) malloc(sizeof(REAL) * (len+4));
    if (argc > 1)
        fn = argv[1];
    else
        fn = "result65497_1000";
    f = fopen(fn, "wt");
    for (snr = (MIN_SNR*2); snr <= (MAX_SNR*2); snr++) {
        UINT32 sum[it_num], t;
        UINT16 it;

#ifdef TURBO_ORIG
        ch_setEbN0(0.5 * snr, 1.0/2);
#else
        ch_setEbN0(0.5 * snr, 1.0/3);
#endif
        assert(ch_getSigma() > 0);

        for (it = 0; it < it_num; it++)
            sum[it] = 0;

        for (i = 0; i < MAX_TRIAL; i++) {
            int last_it = 0;
            for (t = 0; t < 3; t++)
                ch_BIT2AWGN(y[t], r[t], len+4);
#ifdef TURBO_ORIG
            for (t = 1; t < len+4; t+=2) {
                 r[2][t-1] = 0;
                 r[2][t] = r[1][t];
                 r[1][t] = 0;
            }
            if ((len+4)&1)
                 r[2][len+4-1] = 0;
#endif
            turbo_decode_sub(r, 0, 0, 0, 0, len, 4, 0);
            for (t = 0; t < len+4; t++) pi[t] = 0;

            for (it = 0; it < it_num; it++) {
                turbo_decode_sub(r, pi, LLR, iters[it] - last_it,
                                 ch_getSigma(), len, 4, 1);
                last_it = iters[it];
                sprintf(pdfn, "%s_%2.1f_%d_pi", fn, 0.5*snr, iters[it]);
                dot = strchr(pdfn, '.'); *dot = '_';
                fpi = fopen(pdfn, "at");
                sprintf(pdfn, "%s_%2.1f_%d_LLR", fn, 0.5*snr, iters[it]);
                dot = strchr(pdfn, '.'); *dot = '_';
                fLLR = fopen(pdfn, "at");

                for (t = 0; t < len; t++) {
                    BIT u = (LLR[t] < 0);
                    if (u != x[t])
                        sum[it]++;
                    fprintf(fpi, "%lf\n", pi[t] * (1-x[t]-x[t]));
		    fprintf(fLLR, "%lf\n", LLR[t] * (1-x[t]-x[t]));
                }
		fclose(fpi); fclose(fLLR);
            }
            turbo_decode_sub(r, 0, 0, 0, 0, len, 4, 2);
        }
        printf("Eb/N0 = %3.1f dB\n", 0.5*snr);
        for (it = 0; it < it_num; it++) {
            printf("  iters = %d, err# = %8.4lf;\n",
                    iters[it], sum[it] / (REAL)MAX_TRIAL);
            fprintf(f, "%8.4lf  ", sum[it] / (REAL)MAX_TRIAL);
        }
        fprintf(f, "\n");
    }
    fclose(f);
    free(pi);
    free(LLR);
}
